@@ -153,8 +153,12 @@ class Stream(object):
             self.pending_backprocess = bool(all_tags['pending_backprocess'])
             self.latest_datapoint = all_tags['latest_datapoint']
             self.earliest_datapoint = all_tags['earliest_datapoint']
-            self.downsampled_until = all_tags['downsampled_until']
             self.value_type = all_tags['value_type']
+            self.value_type_options = all_tags['value_type_options']
+
+            if 'downsampled_until' in all_tags:
+                self.downsampled_until = all_tags['downsampled_until']
+                del self.tags['downsampled_until']
 
             if 'derived_from' in all_tags:
                 self.derived_from = all_tags['derived_from']
@@ -170,8 +174,8 @@ class Stream(object):
             del self.tags['pending_backprocess']
             del self.tags['latest_datapoint']
             del self.tags['earliest_datapoint']
-            del self.tags['downsampled_until']
             del self.tags['value_type']
+            del self.tags['value_type_options']
         except KeyError, exception:
             raise ValueError("Supplied tags are missing %s." % exception)
 
@@ -187,6 +191,7 @@ RESERVED_TAGS = (
     'earliest_datapoint',
     'downsampled_until',
     'value_type',
+    'value_type_options',
 )
 
 VALUE_DOWNSAMPLERS = {
@@ -222,14 +227,17 @@ DERIVE_OPERATORS = {
     'counter_derivative': 'COUNTER_DERIVATIVE', # derivative of a monotonically increasing counter stream
 }
 
-VALUE_TYPES = {
+VALUE_TYPES = (
     'numeric',
     'graph',
     'nominal',
-}
+)
 
 
 class ResultsBase(object):
+    def __init__(self, *args, **kwargs):
+        pass
+
     def batch_size(self, batch_size): # pylint: disable=unused-argument
         # Ignore by default, this is just for optimization
         return
@@ -266,6 +274,7 @@ class Datastream(object):
     VALUE_DOWNSAMPLERS = VALUE_DOWNSAMPLERS
     TIME_DOWNSAMPLERS = TIME_DOWNSAMPLERS
     DERIVE_OPERATORS = DERIVE_OPERATORS
+    VALUE_TYPES = VALUE_TYPES
 
     def __init__(self, backend):
         """
@@ -296,7 +305,7 @@ class Datastream(object):
 
         self._check_query_tags(tags)
 
-    def ensure_stream(self, query_tags, tags, value_downsamplers, highest_granularity, derive_from=None, derive_op=None, derive_args=None, value_type=None):
+    def ensure_stream(self, query_tags, tags, value_downsamplers, highest_granularity, derive_from=None, derive_op=None, derive_args=None, value_type=None, value_type_options=None, derive_backprocess=True):
         """
         Ensures that a specified stream exists.
 
@@ -310,6 +319,8 @@ class Datastream(object):
         :param derive_op: Derivation operation
         :param derive_args: Derivation operation arguments
         :param value_type: Optional value type (defaults to `numeric`)
+        :param value_type_options: Options specific to the value type
+        :param derive_backprocess: Should a derived stream be backprocessed
         :return: A stream identifier
         """
 
@@ -337,8 +348,15 @@ class Datastream(object):
             value_type = 'numeric'
         if value_type not in VALUE_TYPES:
             raise exceptions.UnsupportedValueType("Unsupported value type: %s" % value_type)
+        if value_type_options is None:
+            if value_type == 'numeric':
+                value_type_options = {'high_accuracy': False}
+            else:
+                value_type_options = {}
+        elif not isinstance(value_type_options, dict):
+            raise TypeError("Value type options must be a dictionary")
 
-        return self.backend.ensure_stream(query_tags, tags, value_downsamplers, highest_granularity, derive_from, derive_op, derive_args, value_type)
+        return self.backend.ensure_stream(query_tags, tags, value_downsamplers, highest_granularity, derive_from, derive_op, derive_args, value_type, value_type_options, derive_backprocess)
 
     def get_tags(self, stream_id):
         """
@@ -398,6 +416,17 @@ class Datastream(object):
 
         return self.backend.find_streams(query_tags)
 
+    def append_multiple(self, datapoints):
+        """
+        Appends multiple datapoints into the datastream. Each datapoint should be
+        described by a dictionary with fields `stream_id`, `value` and `timestamp`,
+        which are the same as in `append`.
+
+        :param datapoints: A list of datapoints to append
+        """
+
+        return self.backend.append_multiple(datapoints)
+
     def append(self, stream_id, value, timestamp=None, check_timestamp=True):
         """
         Appends a datapoint into the datastream.
@@ -434,7 +463,7 @@ class Datastream(object):
 
         # TODO: Do we want to allow user to specify order of datapoints returned?
 
-        if start is None == start_exclusive is None:
+        if (start is None) == (start_exclusive is None):
             raise AttributeError("One and only one time range start must be specified.")
 
         if end is not None and end_exclusive is not None:
@@ -467,7 +496,7 @@ class Datastream(object):
 
         return self.backend.get_data(stream_id, granularity, start, end, start_exclusive, end_exclusive, reverse, value_downsamplers, time_downsamplers)
 
-    def downsample_streams(self, query_tags=None, until=None, return_datapoints=False):
+    def downsample_streams(self, query_tags=None, until=None, return_datapoints=False, filter_stream=None):
         """
         Requests the backend to downsample all streams matching the specified
         query tags. Once a time range has been downsampled, new datapoints
@@ -479,6 +508,7 @@ class Datastream(object):
         :param return_datapoints: Should newly downsampled datapoints be returned, this can
                                   potentially create a huge temporary list and memory consumption
                                   when downsampling many streams and datapoints
+        :param filter_stream: An optional callable which returns false for streams that should be skipped
         :return: A list of dictionaries containing `stream_id`, `granularity`, and `datapoint`
                  for each datapoint created while downsampling, if `return_datapoints` was set
         """
@@ -488,7 +518,7 @@ class Datastream(object):
         if until is not None and until.tzinfo is None:
             until = until.replace(tzinfo=pytz.utc)
 
-        return self.backend.downsample_streams(query_tags, until, return_datapoints)
+        return self.backend.downsample_streams(query_tags, until, return_datapoints, filter_stream)
 
     def backprocess_streams(self, query_tags=None):
         """
